/* Copyright (C) 2009 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INFLATE_INFLATE_H_INCLUDED
#define INFLATE_INFLATE_H_INCLUDED

#include <stdio.h>

#include "huffman.h"

int
inflate_stream(FILE *in, FILE *out);

#define INFLATE_WINDOW_MAX 32768
#define INFLATE_WINDOW_MASK (INFLATE_WINDOW_MAX - 1)
#define INFLATE_HLIT_MAX 286
#define INFLATE_HDIST_MAX 30
#define INFLATE_HCLEN_MAX 19
#define INFLATE_CLEN_MAX (INFLATE_HLIT_MAX + INFLATE_HDIST_MAX)

enum inflate_status {
/* Set while inflating, cleared when the final byte has been
   processed. */
  INFLATE_FLAG_INPUT_EXPECTED = 1,
  /* Set when there's more output corresponding to th einptu that has
     already been processed. */
  INFLATE_FLAG_OUTPUT_PENDING = 2,

  INFLATE_FLAG_ERROR = 4
};

struct inflate_ctx
{
  /* A combination of INFLATE_FLAG_* values. When inflation is
     completed, status is 0. */
  enum inflate_status status;

  const char *error;

  /* Implementation private variables follow. */
  
  /* FIXME: Could hide these for users, but then we'd need a function
     that returns the allocation size. If that's useful depends on
     what level of binary compatibility we need and how often the
     private stuff is expected to change. */

  /* Input state: Bits read. */
  struct bit_buffer input_buffer;

  /* Output state: A circular buffer. */
  unsigned output_end;
  /* Smallest of INFLATE_WINDOW_MAX and the amount of generated output. */
  unsigned output_size;
  char output_window[INFLATE_WINDOW_MAX];

  int state;

  int final;

  /* Last code word, used when the code is followed by additional bits. */
  unsigned code;
  
  /* For data to be copied (uncompressed data or from the output window). */
  unsigned length;

  /* For compressed blocks. */
  struct huffman_node length_tree[HUFFMAN_SIZE(INFLATE_HLIT_MAX)];
  struct huffman_node distance_tree[HUFFMAN_SIZE(INFLATE_HDIST_MAX)];
  struct huffman_ctx length_active;
  struct huffman_ctx distance_active;

  /* For dynamic coding */
  unsigned hlit;
  unsigned hdist;
  unsigned hclen;
  unsigned char clen[INFLATE_CLEN_MAX];
  unsigned k;
};

void
inflate_init (struct inflate_ctx *self);

/* The provided output will point either into the internal buffer, or
   into the input area. So it must be taken care of before the input
   is overwritten. */
unsigned
inflate (struct inflate_ctx *self, unsigned input_length, const char *input,
	 unsigned *output_length, const char **output);

#endif /* INFLATE_INFLATE_H_INCLUDED */
